#ifndef LINT
static char *sccs_sid = "@(#)rdseq.c	2.1 3/21/89 FAULKNLIB";
#endif

#include <stdio.h>
#define TRUE 1
#define FALSE 0
#define MAX_SEQ 200000

/*STARTDOC*/
/* this unit contains the subroutines:
    rdseq   to ask user for name of file and locus, then fetches it,
            and splices
    next_locus   gets next locus contained in an open file, puts
                 file pointer at start of sequence
    get_sequence gets next part of sequence in an opened file
    write_sequence output sequence - put into 70 chars per line

    rdseq (filnam, afnam, locus, alocus, max, len, seq, frgdsc, all)
    char *filnam, *locus, *seq, *frgdsc;
    int afnam, alocus, max, *len, all;

    next_locus (finp, locus)
    char *locus;
    FILE *finp;

    get_sequence (inseq, finp, len, max)
    char *inseq;
    FILE *finp;
    int *len, max;

    write_sequence (seq, fout)
    char *seq;
    FILE *fout
*/
/*ENDDOC*/
#define min(x, y)	(((x) < (y)) ? (x) : (y))

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

/*STARTDOC*/
rdseq (filnam, afnam, locus, alocus, max, len, seq, frgdsc, all)
char *filnam, *locus, *seq, *frgdsc;
int afnam, alocus, max, *len, all;
/*  filnam inputs default file name, outputs actual file name used.
    if alocus == 0 and filnam is not a null string, the user will not
      be prompted for a new file name.
    locus inputs default file name, outputs actual file name used.
    if alocus == 0 and filnam is not a null string, the user will not
      be prompted for a new file name.
    max inputs the maximum sequence that can be read in.
    len returns the length of the sequence as extracted
    seq is the array that returns the input sequence
    frgdsc contains character string defining extracted string

    comments are completely ignored
*/
/*ENDDOC*/
{   char inlocus [20], tstr [40], tstr1 [10];
    char *inseq, *dflt;
    int temp, start, stop, index;
    FILE *finp;

    *len = 0;
    inseq = (char *) emalloc (MAX_SEQ);

get_info:

  do
  { dflt = (*filnam == '\0') ? NULL : filnam;
    if ((afnam != 0) || (filnam [0] == 0))
      get_response (filnam, 
	"Enter name of file containing query\n  sequence", dflt);
    if ((finp = fopen (filnam, "r")) == NULL)
    { printf ("ERROR - file %s could not be opened - reenter.\n\n", filnam);
      filnam [0] = 0;
    }
  } while (finp == NULL);

  dflt = (*locus == '\0') ? NULL : locus;
  if ((alocus != 0) || (locus [0] == 0))
    get_response (locus, "Enter locus name of query sequence", dflt);

  do
  { temp = next_locus (finp, inlocus);
  } while ((strcmp (inlocus, locus) != 0) && temp);

  if (temp)
/* locus was found, read in the sequence */
  { get_sequence (inseq, finp, len, MAX_SEQ);
    fclose (finp);
  }
  else
/* locus not found in this file */
  { printf ("\nERROR - locus %s not found in %s.\n\n", locus, filnam);
    alocus = 1; afnam = 1;
    fclose (finp);
    goto get_info;
  }

/* locus found */

get_bound:
  printf ("\n\n%d bases found for locus %s in file %s\n\n", *len, locus,
    filnam);

  if (all == 0)
  {
    printf ("Extracting fragment of interest.\n");
    strcpy (tstr, "1 ");
    index = min (*len, max);
    sprintf (tstr1, "%d", index);
    strcat (tstr, tstr1);
    get_response (tstr, "Input boundaries sep. by a space", tstr);
    sscanf (tstr, "%d %d", &start, &stop);
    if ((stop == *len) && (start == 1))
      sprintf (frgdsc, "Entire sequence, length %d", *len);
    else
      sprintf (frgdsc, "%d to %d", start, stop);
  }
  else
  { start = 1;
    stop = min (*len, max);
    if (stop == *len)
      sprintf (frgdsc, "Entire sequence, length %d", *len);
    else
      sprintf (frgdsc, "1 to %d", stop);
  }
  start--; stop --;  /* because c arrays start at 0 */
  if ((stop - start + 1) > max)
  { printf ("ERROR - code accepts maximum of %d bases - reenter.\n\n", max);
    goto get_bound;
  }
  *len = stop - start + 1;
  for (index = start; index <= stop; index++)
    seq [index - start] = inseq [index];
  free (inseq);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

/*STARTDOC*/
next_locus (finp, locus)
char *locus;
FILE *finp;
/* advances pointer on finp to point to the next sequence.
   pointer is left pointing to the first line past the locus line.
   locus is returned containing the locus name.
   returns TRUE if a locus was found,
           FALSE if the end of the file was met before a locus was 
	   found
   ASSUMES that there will be at least one comment line before the 
     locus name appears
*/
/*ENDDOC*/
{ int ptr, temp;
  char line [300], *pt1;

/* search till we find a comment line */
  do
  { if (fgets (line, 300, finp) == NULL)
      return (FALSE);
  } while ((*line != ';') && (*line != 0xc));

  do
  { if (fgets (line, 300, finp) == NULL)
      return (FALSE);
  } while ((line [0] == ';') || (line [0] <= ' '));

/* line contains first line that does not begin with control char, space,
   nor ; */

/* assume that line now contains locus name */
  for (pt1 = line; *pt1 > ' '; pt1++)
  {   *locus = *pt1;
      locus++;
  }
  *locus = '\0';
  return (TRUE);   /* return saying we found a locus */
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

/*STARTDOC*/
get_sequence (inseq, finp, len, max)
char *inseq;
FILE *finp;
int *len, max;
/* reads the remainder of a sequence, up to max length.
   returns TRUE if some sequence was read,
           FALSE if no sequence information was found
*/
/*ENDDOC*/
{ int temp, start, ch2;
  char line [100], ptr;

  max -= 70;  /* so that reading in that last line will not
                 overrun the buffer */

  start = *len;

  ch2 = 65;
  do
  {  ch2 = getc (finp);
     if ((ch2 == ';') || (ch2 == 0xc) || (ch2 == EOF))
     { ungetc (ch2, finp);
       return (start != *len); /* return FALSE if EOF */
     }
     if (ch2 != '\n')
     {   line [0] = ch2;
	 if (!(fgets (&(line [1]), 100, finp)))
             return (start != *len); /* return FALSE if EOF */
	 for (ptr = 0; (temp = line [ptr]) != 0; ptr++)
	 if (temp > ' ')
	 {   *inseq = temp;
	     inseq++;
             (*len)++;
	 }
         if (*len > max)
/* we have an overrun */
             return (TRUE);
     }
  } while (1 == 1);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

int add_one_if_needed = 1;

/*STARTDOC*/
write_sequence (seq, fout)
char *seq;
FILE *fout;
/*ENDDOC*/
{   int ix;

    for (ix = 70; *seq; seq++)
    {   putc (*seq, fout);
	ix--;
	if (!ix)
	{   putc ('\n', fout);
	    ix = 70;
	}
    }
    seq--;
    if ((*seq != '1') && (*seq != '2') && add_one_if_needed)
        fprintf (fout, "1\n");
    else
	if (ix != 70)
	    putc ('\n', fout);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   


