#ifndef LINT
static char *sccs_sid = "@(#)string_subst.c	2.1 3/21/89 FAULKNLIB";
#endif

/*STARTDOC*/
/*  this unit contains a set of subroutines that deal with alias
    substitution.  They are:

    char *get_alias (string, aliases, substitutes)
	expands the alias "string" - returns it's substitution.
	if it is not found, returns NULL.
	upon return, pgm MUST NOT modify the the substituted string
    add_alias (new_alias, new_substitute, aliases, substitutes)
	adds a new alias to the lists.
	if alias exists, it the new will replace the old
	assumes that the lists are large enough.
	allocates space for the new entries via malloc

    char *alias_expand (string, aliases, substitutes)
	returns pointer to string which has aliases replaced
	of string - - aliases introduced with "ALIAS_CHAR", terminated
	by space or EOS, or "ALIAS_CHAR" - RECURSIVE ! ! ! 
	returned string allocated via malloc () - free when finished 
    show_aliases (aliases, substitutes)
	display current aliasing
*/
/*ENDDOC*/

#include <stdio.h>
#include "faulkn.h"

#define MAX_PATCH 2000
    /*  longest string that alias_expand can handle (output)  */
#define MAX_ALIAS 200
    /*  longest possible alias name  */

char ALIAS_CHAR = '$';
    /*  character to introduce and separate alias names  */
char QUOTE_CHAR = '\\';
    /*  literal next  */

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/*STARTDOC*/
char *get_alias (string, aliases, substitutes)
/*  expands the alias "string" - returns it's substitution.
    if it is not found, returns NULL.
    upon return, pgm MUST NOT modify the the substituted string
*/
char **aliases, **substitutes;
/*ENDDOC*/
{   
    while (*aliases)
    {   if (strcmp (*aliases, string) == 0)
	    return (*substitutes);
	aliases++;
	substitutes++;
    }
    return (NULL);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/*STARTDOC*/
add_alias (new_alias, new_substitute, aliases, substitutes)
/*  adds a new alias to the lists.
    if alias exists, it the new will replace the old
    assumes that the lists are large enough.
    allocates space for the new entries via malloc
*/
char *new_alias, *new_substitute, **aliases, **substitutes;
/*ENDDOC*/
{   char *new_string (), *tmp;

    while (*aliases)
    {   if (strcmp (*aliases, new_alias) == 0)
	{   free (*substitutes);
	    *substitutes = new_string (new_substitute);
	    return (1);
	}
	aliases++;
	substitutes++;
    }
    *aliases = new_string (new_alias);
    tmp = *substitutes = new_string (new_substitute);
    while (*tmp)
	tmp++;
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/*STARTDOC*/
char *alias_expand (string, aliases, substitutes)
/*  returns pointer to string which has aliases replaced
    of string - - aliases introduced with "ALIAS_CHAR", terminated
    by space or EOS, or "ALIAS_CHAR" - RECURSIVE ! ! ! 
    returned string allocated via malloc () - free when finished 
*/
char *string, **aliases, **substitutes;
/*ENDDOC*/
{   char *out_string;
    char *ptr, *ptr2, *ptr3, tmp [MAX_ALIAS];

    out_string = (char *) emalloc (MAX_PATCH);
    ptr = out_string;
    *ptr = '\0';
    while (*string)
    {   if (*string == ALIAS_CHAR)
	{   string++;  /*  step past ALIAS_CHAR  */
	    for (ptr2 = tmp; ((*string != '\0') && 
		(*string != ALIAS_CHAR) && 
		(*string > ' ')); string++)
	    {   *ptr2 = *string;
		ptr2++;
	    }
	    *ptr2 = '\0';
	    if (*string == ALIAS_CHAR)
		string++;
	    if (*tmp)
	    {   ptr2 = get_alias (tmp, aliases, substitutes);
		if (ptr2 == NULL)
		/*  search failed  */
		{   fprintf (stderr, "No alias for \"%s\"", tmp);
		    fprintf (stderr, ", no substitution\n");
		    ptr2 = tmp;
		}
		ptr3 = alias_expand (ptr2, aliases, substitutes);
		strcpy (ptr, ptr3);
		free (ptr3);
		while (*ptr)
		   ptr++;
	    }
	}
	else
	{   if (*string == QUOTE_CHAR)
		string++;  /*  step past QUOTE_CHAR  */
	    *ptr = *string;
	    ptr++;
	    if (*string)
		string++;
	    *ptr = '\0';
	}
    }
    return (out_string);
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
/*STARTDOC*/
show_aliases (aliases, substitutes)
/*  Display the current aliases for the user to see  */
char **aliases, **substitutes;
/*ENDDOC*/
{    char **ali, **subst;

    ali = aliases; subst = substitutes;

    printf ("\nCurrent aliases:\n");
    while (*ali)
    {   printf ("    \"%s\" -> \"%s\"\n", *ali, *subst);
	ali++; subst++;
    }
    printf ("\n");
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */

