#ifndef LINT
static char *sccs_sid = "@(#)w-open-file.c	2.1 3/21/89 WINLIB";
#endif

#include "curses.h"
#include <sys/file.h>


FILE *nread (), *w_read_file (), *w_write_file ();

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/*STARTDOC*/
FILE *w_open_file (name, mode, history)
char *name, *mode, **history;
/*  returns file pointer opened as requested
    name is default file name.  if name is null, user is prompted.
    name is modified to reflect the actual name of the file opened.
    if name is a non-empty string, an open will be attempted
	before user is prompted
    o_mode is "w" or "r" (only first letter significant);
    if o_mode is "r", user continually prompted till file opened.
    if o_mode is "w", user is asked whether to append or overwrite
        if file already exists.
    file name can be:
	1) a file name
	2) name of a pipe (first character is ``!'')
	3) network file (first character is ``:'',
	     and ``:'' separates host name from 
	     remote file name)
*/
/*ENDDOC*/
{   extern int interactive;
    FILE *ftemp;

    if (!interactive)
    {   ftemp = fopen (name, mode);
	if (ftemp == NULL)
	{   move (LINES - 1, 0);
	    printw ("Fatal batch error: could not open \"%s\"",
		name);
	    refresh ();
	    endwin ();
	    exit (1);
	}
	return (ftemp);
    }

    if (strcmp (mode, "w") == 0)
        return (w_write_file (name, history));
    else if (strcmp (mode, "r") == 0)
        return (w_read_file (name, history));
    endwin ();
    printf ("\n\n\nBANG - w_open_file called with invalid argument \"%s\"\n",		mode);
    printf ("Aborting\n");
    exit (1);
    return (NULL);
}


FILE *w_read_file (name, history)
char *name, **history;
{   char stemp [200], *ick, n_host [200], n_fname [200];
    FILE *file;

    if (!*name)
	win_prompt (stdscr, LINES - 2, "Name of file to read",
	    name, history, 0, "File-Read");

    switch (*name)
    {   case '!':
	    if (file = popen (&(name [1]), "r"))
		return (file);
	    sprintf (stemp, 
		"Could not read pipe \"%s\", read from file",
		name);
	    break;
	case ':':
	    strcpy (stemp, name);
	    for (ick = stemp; *ick; ick++)
		if (*ick == ':')
		    *ick = ' ';
	    *n_host = *n_fname = '\0';
	    sscanf (stemp, "%s%s", n_host, n_fname);
	    if (!*n_host || !*n_fname)
	    {   move (LINES - 1, 0);
		addstr ("Use :host:file_name for net access");
		break;
	    }
	    if (file = nread (n_host, n_fname))
		return (file);
	    break;
	default:
	    if (file = fopen (name, "r"))
		return (file);
	    sprintf (stemp, 
		"Could not read file \"%s\", read from file",
		name);
	    break;
      }
      win_prompt (stdscr, LINES - 1, stemp, name, history,
	  0, "File-Read-Error");
      return (w_read_file (name, history));
}



FILE *w_write_file (name, history)
char *name, **history;
{   char stemp [200], *mode;
    FILE *file;
    static char *list [5] = { "APPEND", "OVER-WRITE",
	"ENTER-NEW-NAME", "BACKUP", NULL };

    mode = "w";
    if (!*name)
	win_prompt (stdscr, LINES - 2, "Name of file to write",
	    name, history, 0, "File-Write");

    switch (*name)
    {   case '!':
	    if (file = popen (&(name [1]), "w"))
		return (file);
	    sprintf (stemp, 
		"Could not write to pipe \"%s\", write to",
		name);
	    break;
	case ':':
	    sprintf (stemp, 
		"Cannot write to file over network, write to");
	    break;
	default:
	    if (!access (name, F_OK))
	    {   sprintf (stemp, "\"%s\" exists", name);
		switch (win_prompt (stdscr, LINES - 2, stemp, 
		    (char *) NULL, list, 1, "File-Exists/"))
		{   case 0:   /*  append  - default  */
			mode = "a";
			break;
		    case 1:   /*  overwrite  */
			break;
		    case 2:   /*  enter new name  */
			win_prompt (stdscr, LINES - 2, "File to write",
			    name, history, 0, "File-Write");
			return (w_write_file (name, history));
			break;
		    case 3:   /*  backup  */
			if (!backup_file (name))  {
			  move (LINES - 2, 0);
			  printw ("ERROR - could not backup file \"%s\"",
				  name);
			  Pause (1);
			  return (w_write_file (name, history));
			}
			/*  leave mode "w"  */
			break;
		}
	    }
	    if (file = fopen (name, mode))
		return (file);
	    sprintf (stemp, "Failed opening \"%s\", write to", name);
	    break;
	}
    win_prompt (stdscr, LINES -1, stemp, name, history, 0,
	"File-Write-Error");
    return (w_write_file (name, history));
}

/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   
/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */   

